/*
    Agent - Event Scheduling  (AES) Library 
    Copyright (C) 2017  Marko Djogatovic

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Zaglavlje aes_proc.h
#ifndef __AES_PROC_H__
#define __AES_PROC_H__

#include "aes.h"

namespace aes {
#ifdef STAMPA
  // Pomocna funkcije - potrebna je zbog zadataka 2
  void stampaj_red();
#endif  
  
  // Klasa FIFO red cekanja
  class red : public agent {
#ifdef STAMPA
    friend void stampaj_red();
#endif
  protected:
    // Broj agenata u redu cekanja
    size_t _broj;
    // Polje pokazivaca na entitete
    agent* _red[max_broj_uredu];
  public:
    // Konstruktor
    red(): _broj(0) {}
    // Smestamo agenta u red cekanja
    void smesti(const agent* a) {
      if( _broj < max_broj_uredu) {
        // Smestamo entitet na kraj reda cekanja
        _red[_broj]=const_cast<agent*>(a);
        // Uvecavamo broj agenta za 1
        _broj++;
      }
      else {
        cerr << "red::smesti - Previse agenata u redu cekanja" << endl;
        exit(1);
      }
    }
    // Vadimo agenta iz reda cekanja
    void vadi() {
      if(_broj>0) {
        for(size_t i=1;i<_broj;++i)
          _red[i-1] = _red[i];
        // Umanjujemo broj agenata iz reda cekanja
        --_broj;
      }
      else {
        cerr << "red::vadi - Nema agenata u redu cekanja" << endl;
        exit(1);
      }
    } 
    // Prednji i zadnji agent u redu cekanja
    agent* prednji() const { return _red[0];       }
    agent* zadnji()  const { return _red[_broj-1]; }
    // Vracamo velicinu reda cekanja
    size_t vrati_velicinu() { return _broj; };
  };
  // Klasa resursa
  class resurs : public agent {
  protected:
    // Tekuci broj zauzetih mesta u resursa
    size_t _broj;
    // Maksimalni broj  mesta u resursu
    size_t _kapacitet;
  public:
    // Konstruktor
    resurs(size_t b):_broj(0),_kapacitet(b) {}
    // Zauzima mesto u resursu
    void zauzmi() { 
      if(_broj<_kapacitet)
        // Uvecavamo broj zauzetih mesta
        ++_broj;
      else {
        cerr << "resurs::zauzmi - Sva mesta u resursu su zauzeta" << endl;
        exit(1);
      }
    }
    // Oslobadja mesto u resursu
    void oslobodi() {  
      if(_broj>0)
        // Umanjujemo broj zauzetih mesta
        --_broj;
      else {
        cerr << "resurs::oslobodi - Nema entiteta u resursu" << endl;
        exit(1);
      }        
    }
    // Raspolozivost resursa (ima makar jedno slobodno mesto u resursu)
    bool raspoloziv() { return _broj<_kapacitet; }
  };  
}
#endif // __AES_PROC_H__